//
//		Yasuharu Okamoto, 2011.8
//
#include "stdafx.h"
#include "common.h"
#include "rnkissM.h"
#include "mcmc_disp.h"

using namespace System;
using namespace cmmn;
using namespace MyRNKISS;
using namespace LogisticRegIndividual;

namespace cmmn {

	double Logit( double p ){
		return  Math::Log(p / (1.0 - p));
	}


	void qSort( double * d, int Lb, int Ub ){
		if (Lb < Ub){
			int p = (Lb + Ub) / 2;
			double c = d[p];
			int i = Lb;
			int j = Ub;
			do {
				while (d[i] < c) i++;
				while (c < d[j]) j--;
				if (i <= j){
					double v = d[i];
					d[i] = d[j];
					d[j] = v;
					i++;
					j--;
				}
			} while (i <= j);
			qSort(d, Lb, j);
			qSort(d, i, Ub);
		}
		return;
	}


	void calc_stat( int n, double * data, double & mean, double & median, double & L95Intvl, double & U95Intvl, double & prob ){
		qSort( data, 0, n-1 );
		double sum = 0.0;
		for (int i = 0; i < n; i++)
			sum += data[i];
		mean = sum / n;
		if ((n % 2) == 0){
				median = 0.5 * (data[(n/2) - 1] + data[n/2]);
		}else{
				median = data[n/2];
		}
		int iw = int(0.95 * n);
		int p_L95 = 0;
		int p_U95 = p_L95 + iw;
		double r = data[p_U95] - data[p_L95];
		int sp = p_L95, ep = p_U95;
		while(ep < n){
			if ((data[ep] - data[sp]) < r){
				p_U95 = ep;
				p_L95 = sp;
				r = data[ep] - data[sp];
			}
			sp++;
			ep = sp + iw;
		}
		L95Intvl = data[p_L95];
		U95Intvl = data[p_U95];

		if (data[n-1] <= 0.0){
			prob = 1.0;
		}else{
			int ipos = n-1;
			while (ipos >= 0) {
				if (data[ipos] > 0.0) {
					ipos--;
				}
				else {
					break;
				}
			}
			prob = (ipos+1) / double(n);
		}

		return;
	}


	double scale_sgm( double sgm ){
		double v = 1.0;
		if (sgm > 0.5){
			v = 1.0 + 9.0 * (sgm - 0.5) / 0.5;
		} else if (sgm < 0.3){
			v = 1.0 / (1.0 + 9.0 * (0.3 - sgm) / 0.3);
		}

		return v;
	}

	double ** gX;
	int * gY;


	void mcmc::set_mcmc(int vn, int vp, int vn_simu, double **vX, int * vY){
		if (b != nullptr){
			for (int i = 0; i <= n_simu; i++){
				delete [] b[i];
			}
			delete [] b;
		}
		if (sgm != nullptr)
			delete [] sgm;
		if (cnt != nullptr)
			delete [] cnt;

		n = vn;
		p = vp;
		n_simu = vn_simu;
		pX = vX;
		pY = vY;
		b = new double*[n_simu+1];
		for (int i = 0; i <= n_simu; i++){
			b[i] = new double[p];
		}
		sgm = new double[p];
		cnt = new int[p];

		return;
	}

	double mcmc::LogL( double * b ){
		double v = 0.0;
		for (int i = 0; i < n; i++){
			double z = 0.0;
			for (int j = 0; j < p; j++)
				z += b[j] * pX[i][j];
			if (pY[i] > 0){
				double ez, pi;
				if (z < 708.0){
					ez = Math::Exp(z);
					pi = ez / (1.0 + ez);
				}else{
					pi = 1.0;
				}
				if (pi > 0.0){
					v += Math::Log(pi);
				}else{
					v += z;
				}
			}else{
				double enz, qi;
				if (-z < 708.0){
					enz = Math::Exp(-z);
					qi = enz / (1.0 + enz);
				}else{
					qi = 1.0;
				}
				if (qi > 0.0){
					v += Math::Log(qi);
				}else{
					v += -z;
				}
			}
		}
		return v;
	}

	void mcmc::checkY( System::IO::StreamWriter ^ fout ){
		fout->WriteLine();
		fout->WriteLine("Y...");
		for (int i = 0; i < n; i++)
			fout->Write(" " + pY[i].ToString());
		fout->WriteLine();
		for (int i = 0; i < n; i++){
			for (int j = 0; j < p; j++)
				fout->Write(" " + pX[i][j].ToString());
			fout->WriteLine();
		}
	}

	void mcmc::do_mcmc(){
		mcmc_disp ^ m_d = gcnew mcmc_disp();
		m_d->Show();

		for (int i_simu = 0; i_simu < n_simu; i_simu++){
			if ((i_simu % 1000) == 0){
				m_d->disp_message( msg + "\ri_simu = " + i_simu.ToString() + "/" + n_simu.ToString() + " started.");
				m_d->Update();
			}

			double * tb0, * tb1;
			tb0 = new double[p];
			tb1 = new double[p];
			for (int j = 0; j < p; j++)
				tb1[j] = tb0[j] = b[i_simu][j];
			for (int j = 0; j < p; j++){
				double y = rn->normalMS( tb0[j], sgm[j] );
				tb1[j] = y;
				double L_num = LogL( tb1 );
				double L_den = LogL( tb0 );
				double acpt = Math::Exp( L_num - L_den );
				if (rn->uni() < acpt){
						tb0[j] = tb1[j];
						cnt[j]++;
				}else{
						tb1[j] = tb0[j];
				}
			}
			for (int j = 0; j < p; j++)
				b[i_simu+1][j] = tb1[j];
		}

		m_d->Close();
		return;
	}

}
